import random

from django import template
from django.core.cache import cache
from django.db.models import F

from config.cache_durations import *
from gallery.cache import cached_or_set
from gallery.models import Album, Photo

register = template.Library()


@register.simple_tag
def last_albums(count=5):
    """
    Returns the most recent public albums, limited by 'count'.
    """
    return cached_or_set(
        f"last_albums_{count}",
        LAST_ALBUMS_DURATION,
        lambda: Album.objects.filter(is_public=True).order_by("-album_date")[:count]
    )


@register.simple_tag
def top_photos(count=5):
    """
    Returns top photos from public albums based on favorites, likes, and views.
    """
    return cached_or_set(
        f"top_photos_{count}",
        TOP_PHOTOS_DURATION,
        lambda: (
            Photo.objects
            .filter(album__is_public=True)
            .order_by('-is_favorite', '-likes', '-views')
            .select_related('album')[:count]
        )
    )


@register.simple_tag
def random_photos_landscape(count=5):
    """
    Returns random landscape-oriented photos from public albums.
    Does not use cache; always returns new results.
    """
    all_ids = list(
        Photo.objects
        .filter(album__is_public=True, width__gt=F('height'))
        .values_list('id', flat=True)[:500]
    )
    selected_ids = random.sample(all_ids, min(count, len(all_ids)))
    return Photo.objects.filter(id__in=selected_ids)


@register.simple_tag
def random_favorite_photos_landscape(count=5):
    """
    Returns random landscape-oriented favorite photos from public albums.
    If fewer than 'count' photos exist, returns an empty list.
    Results are cached for RANDOM_FAVORITES_DURATION.
    """
    key = f"random_favorite_photos_landscape_{count}"

    def fetch():
        queryset = Photo.objects.filter(
            is_favorite=True,
            width__gt=F('height'),
            album__is_public=True
        )
        if queryset.count() < count:
            return []
        return list(queryset.order_by('?')[:count])

    return cached_or_set(key, RANDOM_FAVORITES_DURATION, fetch)


@register.simple_tag
def random_favorite_photos_portrait(count=5):
    """
    Returns random portrait-oriented favorite photos from public albums.
    If fewer than 'count' photos exist, returns an empty list.
    Results are cached for RANDOM_FAVORITES_DURATION.
    """
    key = f"random_favorite_photos_portrait_{count}"

    def fetch():
        queryset = Photo.objects.filter(
            is_favorite=True,
            height__gt=F('width'),
            album__is_public=True
        )
        if queryset.count() < count:
            return []
        return list(queryset.order_by('?')[:count])

    return cached_or_set(key, RANDOM_FAVORITES_DURATION, fetch)


@register.simple_tag
def top_photos_landscape(count=5):
    """
    Returns top landscape-oriented photos from public albums, sorted by views.
    Uses select_related for optimization.
    Results are cached for TOP_PHOTOS_DURATION.
    """
    return cached_or_set(
        f"top_photos_landscape_{count}",
        TOP_PHOTOS_DURATION,
        lambda: (
            Photo.objects
            .filter(album__is_public=True, width__gt=F('height'))
            .order_by('-views')
            .select_related('album')[:count]
        )
    )


@register.simple_tag
def top_photos_portrait(count=5):
    """
    Returns top portrait-oriented photos from public albums, sorted by views.
    Uses select_related for optimization.
    Results are cached for TOP_PHOTOS_DURATION.
    """
    return cached_or_set(
        f"top_photos_portrait_{count}",
        TOP_PHOTOS_DURATION,
        lambda: (
            Photo.objects
            .filter(album__is_public=True, height__gt=F('width'))
            .order_by('-views')
            .select_related('album')[:count]
        )
    )


@register.simple_tag
def top_photos_in_album(album, count=5):
    """
    Returns top photos in a specific album, sorted by favorite, likes, and views.
    Uses album ID in the cache key.
    Results are cached for TOP_PHOTOS_DURATION.
    """
    return cached_or_set(
        f"top_photos_album_{album.id}_{count}",
        TOP_PHOTOS_DURATION,
        lambda: (
            Photo.objects
            .filter(album=album)
            .order_by('-is_favorite', '-likes', '-views')
            .select_related('album')[:count]
        )
    )
